
/*
*	TITLE		Queued serial drivers for FreeRTOS using PIC32MZ
*
*   File        serial_RTOS_isr.S
*
*	VERSION:	0.3b (Beta)
*
*	DATE:		1-Dec-2013
*
*	AUTHOR:		Tom Lafleur
*				lafleur@lafleur.us
*
*	COMMENTS:	You are free to use this source code for any non-commercial
*				use. Please do not make copies of this source code, modified
*				or un-modified, publicly available on the internet or
*				elsewhere without permission. Thanks.
*
*			Copyright 2005-2014   All rights are reserved.
*
********************************************************************************
*
*	CHANGE LOG:
*
*	DATE         REV  DESCRIPTION
*	-----------  ---  ----------------------------------------------------------
*	
*	03-Dec-2013	 0.3b TRL - Port to PIC32MZ processor
*
*
*	Notes:	1)  Tested with FreeRTOS ver 7.6.0 MEB II
*			2)  Support for two serial ports, can easily be expanded for more ports
*			3)  Added Rx error counters to the data structures
*			4)  If using processors with Peripheral pin select, you must define
*               pin's prior to using this driver <-----------------------------------
*			5)  Based on FreeRTOS demo code and my 2007 robotics serial driver
*
*	ToDo:	1)  Check if code is atomic and thread safe
*			2)  Make Rx error counters avilable global
*			3)  Make Tx use DMA if available, if so we will use ring buffers
*               and give a full buffer to DMA system. Queue's are too slow.
*			4)
*
* 	My Notes:
*
*
*
*/

/* FreeRTOS standard interrupt wrapper */


#include <xc.h>
#include <sys/asm.h>
#include "ISR_Support.h"
#include "serial.h"


	.set	nomips16
	.set 	noreorder
	.extern xISRStackTop

#if defined UART1

/* *********** UART 1 *********** */

	.extern vU1RXInterruptHandler
	.extern vU1TXInterruptHandler
	.extern vU1FAULTInterruptHandler

	.global	vU1RXInterruptWrapper
	.global	vU1TXInterruptWrapper
	.global	vU1FAULTInterruptWrapper



// Uart 1 RX Interrupt

	.set	noreorder
	.set 	noat
	.ent	vU1RXInterruptWrapper

vU1RXInterruptWrapper:

	portSAVE_CONTEXT
	jal vU1RXInterruptHandler
	nop
	portRESTORE_CONTEXT

	.end	vU1RXInterruptWrapper


// Uart 1 TX Interrupt

	.set	noreorder
	.set 	noat
	.ent	vU1TXInterruptWrapper

vU1TXInterruptWrapper:

	portSAVE_CONTEXT
	jal vU1TXInterruptHandler
	nop
	portRESTORE_CONTEXT

	.end	vU1TXInterruptWrapper


// Uart 1 Error-Fault Interrupt

	.set	noreorder
	.set 	noat
	.ent	vU1FAULTInterruptWrapper

vU1FAULTInterruptWrapper:

	portSAVE_CONTEXT
	jal vU1FAULTInterruptHandler
	nop
	portRESTORE_CONTEXT

	.end	vU1FAULTInterruptWrapper
	

#endif

#if defined UART2

/* *********** UART 2 *********** */

	.extern vU2RXInterruptHandler
	.extern vU2TXInterruptHandler
	.extern vU2FAULTInterruptHandler

	.global	vU2RXInterruptWrapper
	.global	vU2TXInterruptWrapper
	.global	vU2FAULTInterruptWrapper



// Uart 2 RX Interrupt

	.set	noreorder
	.set 	noat
	.ent	vU2RXInterruptWrapper

vU2RXInterruptWrapper:

	portSAVE_CONTEXT
	jal vU2RXInterruptHandler
	nop
	portRESTORE_CONTEXT

	.end	vU2RXInterruptWrapper


// Uart 2 TX Interrupt

	.set	noreorder
	.set 	noat
	.ent	vU2TXInterruptWrapper

vU2TXInterruptWrapper:

	portSAVE_CONTEXT
	jal vU2TXInterruptHandler
	nop
	portRESTORE_CONTEXT

	.end	vU2TXInterruptWrapper


// Uart 2 Error-Fault Interrupt

	.set	noreorder
	.set 	noat
	.ent	vU2FAULTInterruptWrapper

vU2FAULTInterruptWrapper:

	portSAVE_CONTEXT
	jal vU2FAULTInterruptHandler
	nop
	portRESTORE_CONTEXT

	.end	vU2FAULTInterruptWrapper

#endif

#if defined UART3

/* *********** UART 3 *********** */

	.extern vU3RXInterruptHandler
	.extern vU3TXInterruptHandler
	.extern vU3FAULTInterruptHandler

	.global	vU3RXInterruptWrapper
	.global	vU3TXInterruptWrapper
	.global	vU3FAULTInterruptWrapper



// Uart 3 RX Interrupt

	.set	noreorder
	.set 	noat
	.ent	vU3RXInterruptWrapper

vU3RXInterruptWrapper:

	portSAVE_CONTEXT
	jal vU3RXInterruptHandler
	nop
	portRESTORE_CONTEXT

	.end	vU3RXInterruptWrapper


// Uart 3 TX Interrupt

	.set	noreorder
	.set 	noat
	.ent	vU3TXInterruptWrapper

vU3TXInterruptWrapper:

	portSAVE_CONTEXT
	jal vU3TXInterruptHandler
	nop
	portRESTORE_CONTEXT

	.end	vU3TXInterruptWrapper


// Uart 3 Error-Fault Interrupt

	.set	noreorder
	.set 	noat
	.ent	vU3FAULTInterruptWrapper

vU3FAULTInterruptWrapper:

	portSAVE_CONTEXT
	jal vU3FAULTInterruptHandler
	nop
	portRESTORE_CONTEXT

	.end	vU3FAULTInterruptWrapper

#endif

#if defined UART4

/* *********** UART 4 *********** */

	.extern vU4RXInterruptHandler
	.extern vU4TXInterruptHandler
	.extern vU4FAULTInterruptHandler

	.global	vU4RXInterruptWrapper
	.global	vU4TXInterruptWrapper
	.global	vU4FAULTInterruptWrapper



// Uart 4 RX Interrupt

	.set	noreorder
	.set 	noat
	.ent	vU4RXInterruptWrapper

vU4RXInterruptWrapper:

	portSAVE_CONTEXT
	jal vU4RXInterruptHandler
	nop
	portRESTORE_CONTEXT

	.end	vU4RXInterruptWrapper


// Uart 4 TX Interrupt

	.set	noreorder
	.set 	noat
	.ent	vU4TXInterruptWrapper

vU4TXInterruptWrapper:

	portSAVE_CONTEXT
	jal vU4TXInterruptHandler
	nop
	portRESTORE_CONTEXT

	.end	vU4TXInterruptWrapper


// Uart 4 Error-Fault Interrupt

	.set	noreorder
	.set 	noat
	.ent	vU4FAULTInterruptWrapper

vU4FAULTInterruptWrapper:

	portSAVE_CONTEXT
	jal vU4FAULTInterruptHandler
	nop
	portRESTORE_CONTEXT

	.end	vU4FAULTInterruptWrapper

#endif

